@@ -1,19 +1,18 @@
 ;;; ====================================================================
 ;;;  @Emacs-Lisp-file{
 ;;;     author          = "Nelson H. F. Beebe",
-;;;     version         = "1.20",
-;;;     date            = "01 August 1992",
-;;;     time            = "15:42:38 MDT",
+;;;     version         = "1.22",
+;;;     date            = "07 November 1992",
+;;;     time            = "14:15:42 MST",
 ;;;     filename        = "filehdr.el",
 ;;;     address         = "Center for Scientific Computing
 ;;;                        Department of Mathematics
-;;;                        South Physics Building
 ;;;                        University of Utah
 ;;;                        Salt Lake City, UT 84112
 ;;;                        USA
 ;;;                        Tel: +1 801 581 5254
 ;;;                        FAX: +1 801 581 4148",
-;;;     checksum        = "25899 1496 5796 58473",
+;;;     checksum        = "56692 1560 6199 61500",
 ;;;     email           = "beebe@solitude.math.utah.edu (Internet)",
 ;;;     codetable       = "ISO/ASCII",
 ;;;     keywords        = "checksum, file header",
@@ -74,7 +73,6 @@
 ;;;                        (setq file-header-user-address
 ;;;                          "Center for Scientific Computing
 ;;;                        Department of Mathematics
-;;;                        South Physics Building
 ;;;                        University of Utah
 ;;;                        Salt Lake City, UT 84112
 ;;;                        USA")
@@ -187,6 +185,14 @@
 ;;;
 ;;; NB: Don't forget to update file-header-code-version below!
 ;;;
+;;; Version 1.22 [07-Nov-1992]
+;;;	Install workaround in update-checksum for nasty Emacs bug
+;;;	that resulted in loss of the buffer for large files.
+;;;
+;;; Version 1.21 [04-Oct-1992]
+;;;	Modify update-checksum to supply a final newline if one is
+;;;	lacking, and to remove trailing blank lines.
+;;;
 ;;; Version 1.20 [01-Aug-1992]
 ;;;     Change update-date to use delete-region instead of zap-to-char
 ;;;     so as to leave the kill ring untouched.  Internationalize
@@ -342,7 +348,9 @@
   "Version number of the filehdr.el library, a collection of functions
 for the generation and maintenance of standard file headers.")
 
-(setq file-header-code-version "1.20 [01-Aug-1992]")
+(setq file-header-code-version "1.22 [07-Nov-1992]")
+;;; (setq file-header-code-version "1.21 [04-Oct-1992]")
+;;; (setq file-header-code-version "1.20 [01-Aug-1992]")
 ;;; (setq file-header-code-version "1.19 [29-May-1992]")
 ;;; (setq file-header-code-version "1.18 [25-May-1992]")
 ;;; (setq file-header-code-version "1.17 [25-Mar-1992]")
@@ -841,9 +849,13 @@ in the echo area."
   )
 
 (defun update-checksum ()
-  "Trim trailing whitespace from all lines in the buffer, then update
-the checksum line in the standard file header generated by
-make-file-header, using the checksum program to compute it."
+  "Trim trailing whitespace from all lines in the buffer, ensure that
+the final line is terminated by a newline, remove all trailing blank
+lines, then update the checksum line in the standard file header
+generated by make-file-header, using the checksum program to compute
+it.  The whitespace removal is important because some mail and file
+systems may modify it, and no reasonable text file should ever depend
+upon it."
   (interactive)
   (if (eq system-type 'vax-vms)         ;then VAX VMS
       (progn
@@ -853,14 +865,66 @@ make-file-header, using the checksum program to compute it."
       (widen)
       (goto-char (point-min))
       (replace-regexp "[ \t]+$" "")     ;blank trim lines before checksumming
+      (goto-char (1- (point-max)))      ;make sure last character in the
+      (if (not (looking-at "\n"))       ;buffer is a newline, and
+          (progn                        ;supply one if necessary
+            (forward-char 1)
+            (insert "\n")))
+      (goto-char (point-max))
+      (delete-blank-lines)              ;remove trailing blank lines
       (goto-char (point-min))
-      (call-process-region (point-min) (point-max) "checksum" t t nil)
-      (goto-char (point-min))
-      (re-search-forward "checksum[ \t]*=[ \t]*\"")
+
+      ;; We used to just do
+      ;; (call-process-region (point-min) (point-max) "checksum" t t nil)
+      ;; but that fails if the region is large, leaving us with an
+      ;; empty buffer and lost text.  This bug (which is really a
+      ;; deficiency in GNU Emacs, not an error in the filehdr.el code)
+      ;; first showed up 11 months after filehdr.el was written.  It
+      ;; was reported by David Carlisle
+      ;; <carlisle@computer-science.manchester.ac.uk> on Thu, 29 Oct
+      ;; 1992 11:21:31.  The author (NHFB) reproduced it on
+      ;; 07-Nov-1992, and prepared this fix for it.
+      ;;
+      ;; Instead, we run checksum directly using temporary files, and
+      ;; only if it is successful do we even replace the buffer.  In
+      ;; order to create unique temporary file names, we include the
+      ;; mm:ss part of the current hh:mm:ss time stamp in the file
+      ;; name, keeping the filenames within the 14-character limit of
+      ;; some older UNIX systems, in the form "/tmp/FLHDRmm:ss.xyz"
+      ;;
+      ;; A process id would be better than mm:ss, but Emacs' O/S
+      ;; interface is deficient, and does not provide it.
+      ;;
+      ;; A random number would work here too; (concat (lsh (random t)
+      ;; -8)) could be used in place of (substring
+      ;; (current-time-string) 14 19).  However, that would change the
+      ;; random number seed, which might interfere with other
+      ;; functions that use it.
+
+      (let* ((tmpname (concat "/tmp/FLHDR" (substring (current-time-string) 14 19)))
+	     (oldname (concat tmpname ".old"))
+	     (newname (concat tmpname ".new")))
+	(write-region (point-min) (point-max) oldname nil 0)
+        (kill-buffer (get-buffer-create "*checksum*")) ;delete any current contents
+	(if (= 0 (call-process "checksum" nil (get-buffer-create "*checksum*")
+                               nil oldname newname))
+	    (progn
+	      (delete-region (point-min) (point-max))
+	      (insert-file newname)
+	      (goto-char (point-min))
+	      (re-search-forward "checksum[ \t]*=[ \t]*\"")
+              (kill-buffer "*checksum*")
+	      )
+	  (goto-char (point-min))
+          (display-buffer "*checksum*")
+	  (ding))
+        (delete-file oldname)
+        (delete-file newname))
       )                                 ;end save-restriction
-    )
+    )					;if (eq system-type 'vax-vms)
   )
 
+
 (defun update-date ()
   "Update a file header to set the date to today, and the time to now.
 The header is expected to contain lines like
